/*
 * Controlling MCP3208 ADC by SPI (using spidev driver)
 *
 * Copyright (c) 2007  MontaVista Software, Inc.
 * Copyright (c) 2007  Anton Vorontsov <avorontsov@ru.mvista.com>
 *
 * Modifications 2012 by Claus Knel <info@ckuehnel.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This source is http://sourceforge.net/projects/raspberrypisnip/files/C/mcp3208.c/download
 *
 * 2014/7/30-9/2
 * For getting by CSV type file, this source is modified by Hiroshima-gas TRI (AN).
 *
 * (Example) Compile on Raspberry Pi by gcc mcp3208_25.c -o am
 */

#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>
#include <time.h>

#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))

#define DEBUG 0

// Configuration bits for single-ended channel selection
#define CHANNEL0 0x060000   // Channel 0
#define CHANNEL1 0x064000   // Channel 1
#define CHANNEL2 0x068000   // Channel 2
#define CHANNEL3 0x06C000   // Channel 3
#define CHANNEL4 0x070000   // Channel 4
#define CHANNEL5 0x074000   // Channel 5
#define CHANNEL6 0x078000   // Channel 6
#define CHANNEL7 0x07C000   // Channel 7

static const char *device0 = "/dev/spidev0.0";
static const char *device1 = "/dev/spidev0.1";
static uint8_t mode;
static uint8_t bits = 8;
static uint32_t speed = 500000;
static uint16_t delay;

uint32_t channel;

FILE *fp10;
char *fname10 = "measure.csv";		//各チャンネルのスケーリングデータ
int mchno[8][2];    			//-----Chの番号と、計装信号区分(0-5Vが0で1-5Vが1､4ｰ20mAは250Ωの精密抵抗で1-5Vへ外部で変換
char mname[8][31]; 			//-----測定名称(例　ポンプ電力）　漢字なら15文字まで
char munit[8][21]; 			//-----単位（例 kW kWh Nm3/h Nm3/min kPa kgm/s2 など）
float massign[8][2];			//-----上限下限値の実計測値アサイン

FILE *fp20;
char *fname20 = "intervalsec.csv";	//計測間隔と計測名称のデータ
int intervalsec1;			//0:cronによる分単位以上の間隔での計測、1:秒単位の計測
int intervalsec2;			//秒単位計測の場合の計測間隔(秒）
char measure_name[51];			//計測プロジェクト名称
int spanrev;				//mcp3208の電源電圧をmV単位で指定、span調整用

FILE *fp30;
char *fname30 = "mcp3208_header.txt";		//計測ヘッダファイル

int cenr;				//0:CE0を使用 1:CE1を使用

static void pabort(const char *s)
{
	perror(s);
	abort();
}

static void transfer(int fd,int mchno2,float massign1,float massign2)
{
	uint16_t adc;
	uint8_t ret,i;

	uint8_t tx[3]; 		// 3 bytes will be sent to MPC3208
	uint8_t rx[ARRAY_SIZE(tx)] = {0, };

	float adcmv;

	for (i = 0; i < ARRAY_SIZE(tx); i++)
  	{
      	tx[ARRAY_SIZE(tx)-i-1] = channel%256;
      	channel = channel >> 8;
  	}

//  	printf("Sent bytes    : ");
//  	for (i=0; i < ARRAY_SIZE(tx); i++)
//    		printf("%02X", tx[i]);
//  	printf("\n");

	struct spi_ioc_transfer tr =
	{
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)rx,
		.len = ARRAY_SIZE(tx),
		.delay_usecs = delay,
		.speed_hz = speed,
		.bits_per_word = bits,
	};

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	if (ret < 1)
		pabort("can't send spi message");

//	printf("Received bytes: ");
//	for (i = 0; i < ARRAY_SIZE(tx); i++)
//		printf("%02X", rx[i]);
//	printf("\n");

    adc = (rx[1] << 8) + rx[2];
	adc &= 0xFFF;
//	printf("ADC = %5X (hex)\n", adc);
//	printf("ADC = %5d (dec)\n", adc);
//	printf("ADC Voltage = %g mV\n", (float) adc *spanrev/4096);
//	printf("spanrev = %d\n", spanrev);
// 	printf("%g", (float) adc *spanrev/4096);

	adcmv = adc *spanrev/4096;
	//計測値データは、上下スケーリング計算されて、最も精度の表現できる短い方の書式で出力する(%g)
switch (mchno2){
	case 0:
		printf("%g", (float) ((massign2-massign1)/5)*adcmv/1000+massign1);
		break;
	case 1:
		printf("%g", (float) ((massign2-massign1)/4)*adcmv/1000+(5*massign1-massign2)/4);
		break;
	}
}


int transfer2()
{
int ret = 0;
int fd;
int j;

	switch (cenr) {
		case 0:fd = open(device0, O_RDWR);break;
		case 1:fd = open(device1, O_RDWR);break;
	}

	if (fd < 0)
		pabort("can't open device");

	 // max speed hz
	ret = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if (ret == -1)
		pabort("can't set max speed hz");

	ret = ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed);
	if (ret == -1)
		pabort("can't get max speed hz");

#if DEBUG
	printf("spi device: %s\n", device);

	printf("spi mode: %d\n", mode);
	printf("bits per word: %d\n", bits);
	printf("max speed: %d Hz (%d KHz)\n", speed, speed/1000);
#endif

        time_t timer;
        struct tm *date;
        char str[256];
        timer = time(NULL);
        date = localtime(&timer);
        strftime(str, 255, "%Y/%m/%d %H:%M:%S", date);
        printf("%s,", str);

	for (j=0; j<8; j++){
		switch (j){
			case 0:channel = CHANNEL0;break;
			case 1:channel = CHANNEL1;break;
			case 2:channel = CHANNEL2;break;
			case 3:channel = CHANNEL3;break;
			case 4:channel = CHANNEL4;break;
			case 5:channel = CHANNEL5;break;
			case 6:channel = CHANNEL6;break;
			case 7:channel = CHANNEL7;break;
		}
		transfer(fd,mchno[j][1],massign[j][0],massign[j][1]);
		if ( j < 7 ){
			printf(",");
		}else{
			printf("\n");
		}
	}
	close(fd);
}

//計測ヘッダ作成
int makeheader(){

  char *str30 = "chnr,mode,measure-point,unit,low-assign,high-assign\n";
  char *str31 = "* mode 0: 0-5V  , mode 1: 1-5V\n";
  int i10;

  fp30 = fopen(fname30,"w");
  if (fp30 == NULL ){
	printf("%sファイルが開けません\n",fname30);
	return -1;
  }

  fprintf(fp30, "%s",&measure_name[0]);
  fputs("\n",fp30);
  fputs("\n",fp30);

  fputs (str30,fp30);
  for (i10 = 0; i10 < 8; i10++)
	{
  	fprintf(fp30, "%d,%d,%s,%s,%.3f,%.3f", mchno[i10][0], mchno[i10][1], &mname[i10][0], &munit[i10][0], massign[i10][0], massign[i10][1]);
  	fputs("\n",fp30);
	}

  fputs("\n",fp30);
  fputs (str31,fp30);
  fputs ("Span revison level:  ",fp30);
  fprintf (fp30, "%d\n",spanrev);
  fputs("\n",fp30);

  fprintf(fp30, "%s,%s,%s,%s,%s,%s,%s,%s,%s\n", "Date", &mname[0][0],&mname[1][0],&mname[2][0],&mname[3][0],&mname[4][0],&mname[5][0],&mname[6][0],&mname[7][0] );
  fprintf(fp30, " ,(%s),(%s),(%s),(%s),(%s),(%s),(%s),(%s)\n", &munit[0][0], &munit[1][0], &munit[2][0], &munit[3][0], &munit[4][0], &munit[5][0], &munit[6][0], &munit[7][0] );

  fclose (fp30);

  return 0;
}


int main(int argc, char *argv[])
{

time_t last = time(0);
time_t next;
int pastSec = 0;
int xcounter = 0;
int ret10,i10;
int ret20;

if (argc !=2) {
        printf ("引数エラー:引数なし\n");
        return -1;
}

cenr = atoi(argv[1]);
if ( cenr < 0 || cenr > 1 ) {
        printf ("引数エラー:引数0 or 1以外\n");
        return -1;
}

chdir("/usr/local/analog_measure");

fp10 = fopen( fname10, "r" );
  if( fp10 == NULL ){
    printf( "%sファイルが開けません\n", fname10 );
    return -1;
  }

for (i10 = 0; i10 < 8; i10++)
        {
 	ret10 = fscanf( fp10, "%d,%d,%[^,],%[^,],%f,%f", &mchno[i10][0], &mchno[i10][1], &mname[i10][0],&munit[i10][0], &massign[i10][0], &massign[i10][1]);
        }

fclose (fp10);

fp20 = fopen( fname20, "r" );
  if( fp20 == NULL ){
    printf( "%sファイルが開けません\n", fname20 );
    return -1;
  }

ret20 = fscanf( fp20, "%d,%d,%[^,],%d", &intervalsec1, &intervalsec2, &measure_name[0] ,&spanrev );
if (intervalsec2 >= 60){
  printf("計測間隔設定エラー\n");
  return -1;
}

fclose (fp20);

makeheader();		//計測内容ヘッダ作成

transfer2();		//初回計測

if (intervalsec1 == 1){
  while(1){
	if (xcounter >= 60) return -1;
  	if (time(&next) != last){
     		last = next;
     		pastSec++;
     		xcounter++;
		if (xcounter == intervalsec2){
			transfer2();		//指定秒経過毎の計測
			xcounter = 0;
		}
     		if (pastSec == 59 ) return 0;
  	}
  }
}
return 0;
}

