//      Data reading for DHT11 or DHT22(AM2302,RTH03)　sensor
//      trhread.c
//      21,June 2014 by Hiroshima gas TRI (by AN)
//      Original source is made by http://dolls.orz.hm/?p=9000 (spcial thanks)
//      Raspberry Piの/home/piにおいて、このソースコードを置いて、 
//	sudo gcc-o trhsensor trhread.c -L/usr/local/lib -lwiringPi
//      にて、     /home/pi/trhsensorの実行ファイルが作成される。
//      実行は　./trhsensor 11 4 のように、引数を付ける。
//	第1引数は、DHT11の場合11、DHT22(AM2302,RTH03)の場合22。
//	第2引数は、GPIOの番号（RaspberryPi名)(Boardcom名ではない）。
//      cronにて、実行結果をファイルに連続記述させれば、連続した計測データが集積できる。



#include <wiringPi.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <time.h>
#define MAX_TIME 85

int dht_val[5]={0,0,0,0,0};
int res;
int GPIOPIN;
int type = 0;
float f, h;

int dht_read_val()
{
  uint8_t lststate=HIGH;
  uint8_t counter=0;
  uint8_t j=0,i;
  float farenheit;
  for(i=0;i<5;i++)
     dht_val[i]=0;
  pinMode(GPIOPIN,OUTPUT);
  digitalWrite(GPIOPIN,LOW);
  delay(18);
  digitalWrite(GPIOPIN,HIGH);
  delayMicroseconds(40);
  pinMode(GPIOPIN,INPUT);
  for(i=0;i<MAX_TIME;i++)
  {
    counter=0;
    while(digitalRead(GPIOPIN)==lststate){
      counter++;
      delayMicroseconds(1);
      if(counter==255)
        break;
    }
    lststate=digitalRead(GPIOPIN);
    if(counter==255)
       break;
    // top 3 transistions are ignored
    if((i>=4)&&(i%2==0)){
      dht_val[j/8]<<=1;
      if(counter>16)
        dht_val[j/8]|=1;
      j++;
    }
  }
  // verify cheksum and print the verified data
  if((j>=40)&&(dht_val[4]==((dht_val[0]+dht_val[1]+dht_val[2]+dht_val[3])&
0xFF)))
  {
        time_t timer;
        struct tm *date;
        char str[256];
        timer = time(NULL);
        date = localtime(&timer);
        strftime(str, 255, "%Y/%m/%d %H:%M:%S", date);
        printf("%s,", str);

    if (type == 11) {
        printf("%d,%d\n", dht_val[0], dht_val[2]);
        // DHT11の場合は,dht_val[1]とdht_val[3]は全て0なので、無視して、dht_val[0]の湿度とdht_val[2]の温度をダイレクトに表示
        }
    if (type == 22) {
	// float f, h;

	h = dht_val[0] * 256 + dht_val[1];
 	h /= 10;
	f = (dht_val[2] &0x7F) * 256 + dht_val[3];
 	f /= 10;
	if (dht_val[2] &0x80) f *= -1;
	printf("%.1f,%.1f\n", h , f);
        // DHT22の場合は,dht_val[0]とdht_val[1]を連続した2進数として評価し10で割って湿度を計算
        // dht_val[2]の下位7bitとdht_val[3]を連続した2進数として評価し10で割って温度を計算し、
	// dht_val[2]の上位1bitで1ならば氷点下、0ならば0度以上と判断
    }
    return 2;
  }
}

int main(int argc,char *argv[])
{
if (argc != 3) {
	printf ("引数エラー(1)\n");
	return 3;
}
if (strcmp(argv[1], "11") == 0) type = 11;
if (strcmp(argv[1], "22") == 0) type = 22;
if (type == 0) {
	printf ("引数エラー(2)\n");
	return 3;
}
GPIOPIN = atoi(argv[2]);
if ((GPIOPIN < 0) || (GPIOPIN >= 8)) {
	printf ("引数エラー(3)\n");
	return 3;
}
if(wiringPiSetup()==-1)
    exit(1);
  while(1)
  {
     res = 1;
     res = dht_read_val();
     if(res==2) {
      return 0;
      break;
     }
     delay(3000);
  }
  return 1;
}
